{{
Telnet client and server example by Roger Williams localroger@yahoo.com

This is a minimal example showing how to use the api_telnet_serial driver
by Harrison Pham.  It is intended to show the basics of building a client
and server either together or individually.

In the course of putting this together I discovered a few bugs in the
api_telnet_serial code which would cause it to hang if the connection was
interrupted at the wrong time.  

This example is MIT licensed (see end of code).  NOTE that the api_telnet_serial
driver and its sub-objects by Harrison Pham are distributed under the GNU
general public license, not the MIT license, as noted in their source code.

}}
CON

  _clkmode = xtal1+pll16x
  _xinfreq = 5_000_000

  nic_cs      = 3
  nic_sck     = 4
  nic_si      = 5
  nic_so      = 6
  nic_int     = 7
  nic_xtalout = -1

DAT

  mac_addr      byte    $10, $00, $00, $00, $00, $01       ' propeller's mac address

  ip_addr       byte    192, 168, 2, 21                    ' propeller's ip address
  ip_subnet     byte    255, 255, 255, 0                   ' network subnet
  ip_gateway    byte    192, 168, 2, 254                   ' network gateway (router)
  ip_dns        byte    192, 168, 2, 254                   ' network dns

  ip_serv       byte    192, 168, 2, 20                    ' remote server for propeller client
  port_serv     byte    23                                 ' via telnet port

  port_host     byte    23                                 ' propeller server host port = telnet

VAR

  long myServerIP               ' remote server's IP, packed in a 32 bit number
  word myServerPort             ' remote server port number
  word myHostPort               ' our server host port number

  long ServStack[64]

OBJ

  'note: two different instances of api_telnet_serial!
  'each of these gets its own handle when it connects or listens
  sock    : "api_telnet_serial"  'client
  serv    : "api_telnet_serial"  'server
  
  key     : "keyboard"
  text    : "tv_text" 

pub main

  key.start(26,27)              'demoboard keyboard
  text.start(12)                'demoboard video
  'text.start(16)                'debug video
  
  '    start(cs, sck, si, so, int, xtalout, macptr, ipconfigptr)
  sock.start(nic_cs, nic_sck, nic_si, nic_so, nic_int,-1,@mac_addr,@ip_addr)          
  delay_ms(150)                 'let nic finish initializing
  
  myServerIP := (byte[@ip_serv]<<24)+(byte[@ip_serv+1]<<16)+(byte[@ip_serv+2]<<8)+byte[@ip_serv+3]
  myServerPort := byte[@port_serv]

  myHostPort := byte[@port_host]

  {
  This cognew starts the server.  If you want a server but not a client,
  you can just make this a call to MyServer and let this cog do the duty.
  }
  cognew(MyServer, @ServStack)  'omit this if you don't want server
  delay_ms(150)                 'wait for server cog to start

  'this loop maintains the client

  repeat

    text.str(string("Press Any Key to Connect...",13))
    'key.getkey
    MyClient

{
  This very minimal telnet client might or might not work
  as written with any particular telnet server; it was
  tested with an AnyBus-S embedded card.  I have not tried
  this, but it should be possible to start it (or more than
  one instance of it) via cognew as the server is started
  above in order for this cog to continue with other duties.
}
PUB MyClient | c

    \sock.connect(myServerIP, myServerPort)
    sock.resetBuffers     
    sock.waitConnectTimeout(2000)
     
    if sock.isConnected
      text.str(string("Connected!",13))
      repeat while sock.isconnected == -1
        'set color for keystroke echoes
        text.out($c)
        text.out($7)
        
        repeat while (c := key.key) <> 0
          if c == $c8
            c := 8 'backspace
          elseif c == $d0
            text.hex(sock.isconnected,8)
          sock.tx(c)
          text.out(c)
        
        text.out($c)
        text.out($0)
        'NB sock.rxcheck might receive an abort
        repeat while (c := \sock.rxcheck) > 0
          if c < 32
            if c == 13 or c == 8
              text.out(c)
            elseif c <> $0a
              text.hex(c,2)
              text.out(32)
          elseif c < 128
            text.out(c)
          else
            text.hex(c,2)
            text.out(32)
      text.str(string("Connection lost.",13))
    else
      text.str(string("Failed.",13))

{   
  In order to keep things simple, this server just parrots
  back the total of all hex codes you typed when you hit CR,
  and closes the session when it receives an escape.
}
PUB MyServer | c, ttl
  ' server (runs in a seperate cog)
  ttl := 0
  \serv.listen(MyHostPort)
  repeat
    \serv.rxcheck
    \serv.resetBuffers
    repeat while serv.isConnected
      c := \serv.rxcheck
      if c == 27
        \serv.close
      elseif c == 13
        serv.str(string("Total of bytes sent: "))
        serv.hex(ttl,8)
        serv.tx(13)
        serv.tx(10)
        ttl := 0
      elseif c > 0 
        'serv.tx(c)
        if c <> $0a
          ttl += c  

PRI delay_ms(Duration)
  waitcnt(((clkfreq / 1_000 * Duration - 3932)) + cnt)

{{
                                                   TERMS OF USE: MIT License                                                                                                              

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation     
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    
modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
is furnished to do so, subject to the following conditions:                                                                   
                                                                                                                              
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
                                                                                                                              
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         
}}